module Api
  module Filters
    module Condition

      def self.included(base)
        base.extend(ClassMethods)
      end

      module ClassMethods

        def condition_name(condition_name = nil)
          @condition_name = condition_name if condition_name
          @condition_name
        end

        def condition_symbol(condition_symbol = nil)
          @condition_symbol = condition_symbol if condition_symbol
          @condition_symbol
        end

        def handle?(string)
          string =~ /^#{condition_name}\(.*\)$/
        end

      end

      def initialize(field, condition)
        @field, @condition = field, condition
      end

      def build_param
        { "#{@field} #{self.class.condition_symbol} ?" => extract_param }
      end

      protected
      def extract_param
        query_param = @condition.match(/\((.*)\)/)[1]
        get_value(query_param)
      end

      def get_value(query_param)
        extract_amount(query_param) || extract_int(query_param) || query_param
      end

      def extract_amount(value)
        (value.to_f/100.0) if @field == "total_value"
      end

      def extract_int(value)
        value.to_i if !!(value =~ /^[-+]?[0-9]+$/)
      end

      # def extract_status(value)
      #   Array(value).map { |status| ReportsWorkers::Utils::Status.parse(status) }.flatten if @field == "status"
      # end
      #
      # def extract_payment_method(value)
      #   Array(value).map { |paymentMethod| ReportsWorkers::Utils::PaymentMethod.parse(paymentMethod) }.flatten if @field == "payment_type"
      # end

    end
  end
end
